package org.jeecg.modules.ctop.service.impl;

import cn.com.ctop.common.module.utils.CtopAdConstant;
import cn.com.ctop.kuaishou.modules.utils.LinkUtils;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import org.apache.shiro.SecurityUtils;
import org.jeecg.common.constant.AccountReportConstants;
import org.jeecg.common.system.vo.LoginUser;
import org.jeecg.common.util.DateUtils;
import org.jeecg.common.util.JsonResourceUtil;
import org.jeecg.modules.ctop.mapper.MaterialReportOverViewMapper;
import org.jeecg.modules.ctop.service.IMaterialReportOverViewService;
import org.jeecg.modules.system.service.ISysRoleService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.text.ParseException;
import java.util.*;

@Service
public class MaterialReportOverViewServiceImpl implements IMaterialReportOverViewService {

    @Autowired
    private MaterialReportOverViewMapper materialReportOverViewMapper;

    @Autowired
    private ISysRoleService roleService;

    @Override
    public Set<String> recursiveQuerySubordinate(String userId) {
        Set<String> result;
        //查询当前用户是否存在下级
        Set<String> subordinate = materialReportOverViewMapper.recursiveQuerySubordinateByLeader(userId);
        if (subordinate.isEmpty()) {
            subordinate.add(userId);
            return subordinate;
        } else {
            result = querySubordinate(subordinate, subordinate);
            result.add(userId);
        }
        return result;
    }

    @Override
    public Set<JSONObject> queryProjectBy(Set<String> userIds, int mediaId) {
        JSONArray mediaIds= new JSONArray();
        if (mediaId == CtopAdConstant.PLATFORM_TYPE_BYTEDANCE_INT) {
            mediaIds.add(1);
            mediaIds.add(3);
        }else if (mediaId == CtopAdConstant.PLATFORM_TYPE_KUAISHOU_INT) {
            mediaIds.add(2);
            mediaIds.add(4);
        }
            return materialReportOverViewMapper.queryProjectBy(userIds, mediaIds);
    }

    @Override
    public JSONArray queryProjectIdBy(Set<String> userIds, int mediaId) {
        JSONArray mediaIds= new JSONArray();
        if (mediaId == CtopAdConstant.PLATFORM_TYPE_BYTEDANCE_INT) {
            mediaIds.add(1);
            mediaIds.add(3);
        }else if (mediaId == CtopAdConstant.PLATFORM_TYPE_KUAISHOU_INT) {
            mediaIds.add(2);
            mediaIds.add(4);
        }
        return materialReportOverViewMapper.queryProjectIdBy(userIds, mediaIds);
    }

    @Override
    public Map<String, Object> getTotalModule(int mediaId, String startDate, String endDate, JSONArray projects) {
        Map<String, Object> result = new HashMap<>();
        if (projects.isEmpty()) {
            projects = getProjectsByCurrentUser(mediaId);
        }
        if (mediaId == CtopAdConstant.PLATFORM_TYPE_BYTEDANCE_INT) {
            BigDecimal cost= materialReportOverViewMapper.queryBytedanceCost(startDate, endDate, projects);
            BigDecimal lastCost= materialReportOverViewMapper.queryBytedanceCost(DateUtils.addDay(DateUtils.addDay(endDate,-1),-DateUtils.dateDiff(startDate,endDate)), DateUtils.addDay(endDate,-1),projects);
            result.put("cost", cost);
            result.put("costLink", LinkUtils.countLink(cost==null?BigDecimal.ZERO:cost,lastCost==null?BigDecimal.ZERO:lastCost));
            result.put("video", materialReportOverViewMapper.queryMaterialCount(startDate, endDate, projects));
            result.put("newVideo",materialReportOverViewMapper.queryNewMaterialCount(DateUtils.now(),projects));
            result.put("hot", 0);
            result.put("valid", 0);
        } else if (mediaId == CtopAdConstant.PLATFORM_TYPE_KUAISHOU_INT) {
            BigDecimal cost = materialReportOverViewMapper.queryKuaishouCost(startDate, endDate, projects);
            BigDecimal lastCost= materialReportOverViewMapper.queryKuaishouCost(DateUtils.addDay(DateUtils.addDay(endDate,-1),-DateUtils.dateDiff(startDate,endDate)), DateUtils.addDay(endDate,-1),projects);
            result.put("cost", cost);
            result.put("costLink", LinkUtils.countLink(cost==null?BigDecimal.ZERO:cost,lastCost==null?BigDecimal.ZERO:lastCost));
            result.put("video", materialReportOverViewMapper.queryMaterialCount(startDate, endDate, projects));
            result.put("newVideo",materialReportOverViewMapper.queryNewMaterialCount(DateUtils.now(),projects));
            result.put("hot", 0);
            result.put("valid", 0);
        }
        return result;
    }

    @Override
    public List<JSONObject> getTotalChat(int mediaId, String startDate, String endDate, JSONArray projects) {
        List<JSONObject> result = new ArrayList<>();
        if (projects.isEmpty()) {
            projects = getProjectsByCurrentUser(mediaId);
        }
        try {
            if (mediaId == CtopAdConstant.PLATFORM_TYPE_BYTEDANCE_INT) {
                if (DateUtils.isMoreSixMonth(startDate, endDate)) {
                    result = materialReportOverViewMapper.queryBytedanceChatGroupMonth(startDate, endDate, projects);
                } else {
                    result = materialReportOverViewMapper.queryBytedanceChat(startDate, endDate, projects);
                }
            } else if (mediaId == CtopAdConstant.PLATFORM_TYPE_KUAISHOU_INT) {
                if (DateUtils.isMoreSixMonth(startDate, endDate)) {
                    result = materialReportOverViewMapper.queryKuaishouChatGroupMonth(startDate, endDate, projects);
                } else {
                    result = materialReportOverViewMapper.queryKuaishouChat(startDate, endDate, projects);
                }
            }
        } catch (ParseException e) {
            e.printStackTrace();
        }
        return result;
    }

    @Override
    public JSONObject getTagProportion(int mediaId, String startDate, String endDate, int tagId) {
        return null;
    }

    @Override
    public List<JSONObject> getTopMaterialList(int mediaId, String startDate, String endDate, JSONArray projects) {
        List<JSONObject> result = new ArrayList<>();
        if (projects.isEmpty()) {
            projects = getProjectsByCurrentUser(mediaId);
        }
        if (mediaId == CtopAdConstant.PLATFORM_TYPE_BYTEDANCE_INT) {
            result = materialReportOverViewMapper.queryBytedanceTopMaterial(startDate, endDate, projects);

        } else if (mediaId == CtopAdConstant.PLATFORM_TYPE_KUAISHOU_INT) {
            result = materialReportOverViewMapper.queryKuaishouTopMaterial(startDate, endDate, projects);
        }
        return result;
    }

    @Override
    public PageInfo<JSONObject> getTopMaterialList(int mediaId, String startDate, String endDate, JSONArray projects
            , String channelType,String md5,String target,String order, int pageNo, int pageSize) {
        PageInfo<JSONObject> result = new PageInfo<>();
        if (projects.isEmpty()) {
            projects = getProjectsByCurrentUser(mediaId);
        }
        if (mediaId == CtopAdConstant.PLATFORM_TYPE_BYTEDANCE_INT) {
            String filedAll = JsonResourceUtil.joinAllFiled(AccountReportConstants.dicMapByVideo);
            PageHelper.startPage(pageNo,pageSize,false);
            List<JSONObject> list= materialReportOverViewMapper.queryBytedanceMaterialReport(filedAll,startDate, endDate, projects,md5,target,order);
            Long count= materialReportOverViewMapper.queryBytedanceMaterialReportCount(startDate, endDate, projects,md5);
            result.setTotal(count);
            result.setList(list);
        } else if (mediaId == CtopAdConstant.PLATFORM_TYPE_KUAISHOU_INT) {
            String filedAll = JsonResourceUtil.joinAllFiled(AccountReportConstants.kuaishouVideoReportMap );
            PageHelper.startPage(pageNo,pageSize,false);
            List<JSONObject> list= materialReportOverViewMapper.queryKuaishouMaterialReport(filedAll,startDate, endDate, projects,channelType,md5,target,order);
            Long count= materialReportOverViewMapper.queryKuaishouMaterialReportCount(startDate, endDate, projects,channelType,md5);
            result.setTotal(count);
            result.setList(list);
        }
        return result;
    }

    @Override
    public PageInfo<JSONObject> getTopDesignList(String startDate, String endDate, String userId,String target,String order, int pageSize, int pageNo) {
        PageInfo<JSONObject> result = new PageInfo<>();
        Set<String> subordinate = this.recursiveQuerySubordinate(userId);
        if(subordinate.isEmpty()){
            subordinate=materialReportOverViewMapper.recursiveQuerySubordinateByUserId(userId);
        }
        PageHelper.startPage(pageNo,pageSize,false);
        List<JSONObject> topDesign = materialReportOverViewMapper.queryTopDesign(startDate, endDate, subordinate,target,order);
        Long count=materialReportOverViewMapper.queryTopDesignCount(startDate, endDate, subordinate);
        result.setList(topDesign);
        result.setTotal(count);
        return result;
    }

    @Override
    public JSONObject getMaterialDetailInfo(int mediaId, String md5) {
        JSONObject result = new JSONObject();
        if (mediaId == CtopAdConstant.PLATFORM_TYPE_BYTEDANCE_INT) {
            result= materialReportOverViewMapper.queryBytedanceVideoDetail(md5);
        } else if (mediaId == CtopAdConstant.PLATFORM_TYPE_KUAISHOU_INT) {
            result= materialReportOverViewMapper.queryKuaishouVideoDetail(md5);

        }
        return result;
    }

    @Override
    public Map<String, Object> materialDetailAnalyse(int mediaId, String md5) {
        Map<String,Object> result = new HashMap<>();
        if (mediaId == CtopAdConstant.PLATFORM_TYPE_BYTEDANCE_INT) {
            Long projectId= materialReportOverViewMapper.queryBytedanceProjectIdByMd5(md5);
            result.put("cost",materialReportOverViewMapper.queryBytedanceCostByMd5(md5,projectId));
            result.put("click",materialReportOverViewMapper.queryBytedanceClickByMd5(md5,projectId));
            result.put("materialShow",materialReportOverViewMapper.queryBytedanceMaterialShowByMd5(md5,projectId));
            result.put("play100Rate",materialReportOverViewMapper.queryBytedancePlay100RateByMd5(md5,projectId));
            result.put("likeMaterial",materialReportOverViewMapper.queryBytedanceLikeMaterialByMd5(md5,projectId));
            result.put("commentMaterial",materialReportOverViewMapper.queryBytedanceCommentMaterialByMd5(md5,projectId));
            result.put("shareMaterial",materialReportOverViewMapper.queryBytedanceShareMaterialByMd5(md5,projectId));
            result.put("follow",materialReportOverViewMapper.queryBytedanceFollowByMd5(md5,projectId));
        } else if (mediaId == CtopAdConstant.PLATFORM_TYPE_KUAISHOU_INT) {
            Long projectId= materialReportOverViewMapper.queryKuaishouProjectIdByMd5(md5);
            result.put("charge",materialReportOverViewMapper.queryKuaishouChargeByMd5(md5,projectId));
            result.put("photoShow",materialReportOverViewMapper.queryKuaishouPhotoShowByMd5(md5,projectId));
            result.put("photoClick",materialReportOverViewMapper.queryKuaishouPhotoClickByMd5(md5,projectId));
            result.put("aclick",materialReportOverViewMapper.queryKuaishouAClickByMd5(md5,projectId));
            result.put("bclick",materialReportOverViewMapper.queryKuaishouBClickByMd5(md5,projectId));
            result.put("activation",materialReportOverViewMapper.queryKuaishouActivationByMd5(md5,projectId));
            result.put("play3sRate",materialReportOverViewMapper.queryKuaishouPlay3sRateByMd5(md5,projectId));
        }
        return result;
    }

    @Override
    public List<JSONObject> getMaterialDetailChat(int mediaId, String md5) {
        List<JSONObject> result = new ArrayList<>();
        if (mediaId == CtopAdConstant.PLATFORM_TYPE_BYTEDANCE_INT) {
            result= materialReportOverViewMapper.queryBytedanceVideoChat(md5);
        } else if (mediaId == CtopAdConstant.PLATFORM_TYPE_KUAISHOU_INT) {
            result=  materialReportOverViewMapper.queryKuaishouVideoChat(md5);
        }
        return result;
    }

    //递归查询
    private Set<String> querySubordinate(Set<String> leaderIds, Set<String> result) {
        if (leaderIds.isEmpty()) {
            return result;
        }
        Set<String> temp = materialReportOverViewMapper.recursiveQuerySubordinateByLeaders(leaderIds);
        result.addAll(temp);
        querySubordinate(temp, result);
        return result;
    }

    //当项目为空的时候，根据当前登录人查询可查看的所有项目集合
    private JSONArray getProjectsByCurrentUser(int mediaId) {
        String currentUser = ((LoginUser) SecurityUtils.getSubject().getPrincipal()).getId();
        //TODO 为了解决管理员等角色初始化查全部项目查询效率慢的问题判断角色，待优化
        String roleCode= roleService.getRoleCodeByUserId(currentUser);
        if(roleCode.equals("admin")){
            return null;
        }
        return queryProjectIdBy(recursiveQuerySubordinate(currentUser), mediaId);
    }
}
